/*========================================================================*\

Copyright (c) 1999  Nicolai Langfeldt

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL NICOLAI LANGFELDT, PAUL VOJTA, OR ANYONE ELSE BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

\*========================================================================*/

#include "xdvi-config.h"

#ifdef T1

#include "dvi.h"
#include <tex-file.h>

#include <t1lib.h>
#include <t1libx.h>

/* This is the conversion factor from TeXs pt to TeXs bp (big point).
   Maximum available precision is needed to keep it acurate. */

double bp_fac=72.0/72.27;

/* Try to convert from PS charspace units to DVI units with minimal
   loss of significant digits */
#define t1_dvi_conv(x)	((((int) (x)) << 16)/1000)
/* Convert from DVI units to pt */
#define dvi_pt_conv(x)	(((int) (x)) >> 16)

int find_T1_font(name)

     /* Find the font with the given name */

     char *name;

{
  int i;
  char *filename;

  /* Already loaded? IS THIS NEEDED? XDVI HAS ITS OWN CODE FOR THIS? */
  for (i=T1_Get_no_fonts();i--;i) {
    if (strcmp(T1_GetFontFileName(i),name)==0) {
      /* If we want it we may as well load it */
      if (T1_LoadFont(i)==0) 
	return i;
      else 
	return -1;
    }
  }

  /* Not found.  Locate and load */
  filename=kpse_find_file(name,kpse_type1_format,0);
  if (filename==NULL)
    return -1;

  i=T1_AddFont(filename);

  fprintf(stderr,"Type1 font %s loaded as t1lib font %d\n", name, i);

  if (i>=0) {
    filename=kpse_find_file(name,kpse_afm_format,0);
    if (filename!=NULL) 
      T1_SetAfmFileName(i,filename);

    return i;
  }

  return -1;
}


/* Set character# ch */

#ifdef TEXXET
void set_t1_char (cmd, ch)
     wide_ubyte cmd;
#else
long set_t1_char (ch)
#endif
     wide_ubyte ch;

{
  GC gc;
  int xpos, ypos;
  /* This is the intended font size. FIXME: Where does 1.4 come from? */
  float size=dvi_pt_conv(currinf.fontp->scale) * 1.4 * bp_fac;
  /* How much to scale it? */
  float renderfactor=(pixels_per_inch / shrink_factor) / 100;
  int id=currinf.fontp->t1id;
  int dvi_h_sav;
  long gw;

#ifdef HTEX
  gc = HTeXreflevel > 0? highGC: foreGC;
#else
  gc = foreGC;
#endif
  xpos=PXL_H - currwin.base_x ;

  /* Question, why the + 1 here?  Does it also work out that
     way at sizes larger than 12 and 14?  Yes, so it seems,
     it also appears correct when magnified */
  ypos=PXL_V - currwin.base_y + 1;

  gw = t1_dvi_conv(T1_GetCharWidth(currinf.fontp->t1id,ch))*
    size*renderfactor*shrink_factor;

  if (!gw)

#ifdef TEXXET
  dvi_h_sav = DVI_H;
  if (currinf.dir < 0) DVI_H -= gw;
#endif

#if 0
  fprintf(stderr,"Setting %c(%d/%f) at (%d,%d)\n",ch,gw,size,xpos,ypos);
#endif

#ifdef GREY  /* -- Ends up with multicolor output */
  if (0 && use_grey && !(shrink_factor==1)) {
    T1_AASetCharX(currwin.win, gc, 0, xpos, ypos, 
		  currinf.fontp->t1id, ch, size*renderfactor, NULL);
  } else
#endif

    {
      T1_SetCharX(currwin.win, gc, 1, xpos, ypos, currinf.fontp->t1id,
		  ch, size*renderfactor, NULL);
    }

#ifndef TEXXET
  return gw;
#else
#ifdef Omega
  if ((cmd == PUT1) || (cmd == PUT2))
#else
  if (cmd == PUT1)
#endif
    DVI_H = dvi_h_sav;
  else
    if (currinf.dir > 0) 
      DVI_H += gw;
#endif
}


void read_T1_char(fontp, ch)

     struct font *fontp;
     ubyte ch;

{
  fprintf(stderr,"BUG: asked to load %c\n",ch);
}


void init_t1(void)

{
  /* Initialize t1lib for drawing marks on screen */
  char **vector;
  int font;

  /* FIXME: We want to switch to NO_LOGFILE|IGNORE_CONFIGFILE|
     IGNORE_FONTDATABASE eventually */
  if ((T1_InitLib(IGNORE_CONFIGFILE|IGNORE_FONTDATABASE)==NULL)) {
    fprintf(stderr, "Initialization of t1lib failed\n");
    exit(1);
  }

  /* Xdvi assumes 72 dpi screen.  Or does it? */
  T1_SetDeviceResolutions(72.0*bp_fac,72.0*bp_fac);

#if 0
  vector=T1_LoadEncoding("/home/janl/8r.enc");
  if (vector==NULL) {
    fprintf(stderr, "Cannot find 8r encoding\n");
    exit(1);
  }
#endif 

  T1_SetX11Params(DISP, our_visual, our_depth, our_colormap);
  T1_AASetLevel(T1_AA_HIGH);
  T1_AASetBitsPerPixel(our_depth);
  /* T1_LogicalPositionX(0); /* Origo in upper left corner. */
}

#endif

